@@ -39,7 +39,6 @@ class ShouldIndex:
         if self.crop_indexing_dict['get_crop_point']['bottom_status'] and flag:
             self.crop_indexing_dict['bottom_point'] = True
 
-
     def extension(self):
         self.crop_indexing_dict['extension'] = {
             'status': True
@@ -128,7 +127,6 @@ class ShouldIndex:
 
         self.crop_indexing_dict['bottom_padding_ratio'] = bottom_padding_ratio
 
-
     def crop_head(self):
         """裁剪头"""
         self.crop_indexing_dict['crop_head'] = {
@@ -161,6 +159,15 @@ class ImageCrop:
         self.human_box = None
         self.cloth_box = None
 
+    @staticmethod
+    def __divide(numerator, denominator, default=0):
+        # numerator/denominator
+        # 处理分母为0 的特殊情况，根据不同情况 返回默认值
+        if denominator == 0:
+            return default
+
+        return numerator/denominator
+
     def crop(self, slot_config, img):
         """
         需求文档： https://www.teambition.com/task/619e01f619f251003f289802
@@ -203,7 +210,8 @@ class ImageCrop:
 
         # 左右居中方式  human_center human_and_cloth brand
         x_center_method = crop_info.get(X_CENTER_METHOD, "human_center")
-
+        # 保留上下，还是保留左右，还是兴趣区域上下  human_center human_and_cloth brand
+        retain_x_or_y = crop_info.get(RETAIN_X_OR_Y, "x")
         # 寻找上下裁剪点
         top, bottom, left, right = self._get_top_bottom_crop_point(crop_parameter, img, x_center_method, crop_method)
 
@@ -227,7 +235,130 @@ class ImageCrop:
 
         self.si.get_top_and_bottom_padding(top, padding_top, bottom, padding_bottom)
 
-        if crop_method == 'w':  # 自适应裁剪，slot宽不变,高可以改变
+        # 兴趣区域裁剪
+        # https://thoughts.teambition.com/workspaces/5e37b6adf0f331001a9c172d/docs/625e7db4b587e60001e38a67
+        interest_region_crop = crop_info.get("interest_region_crop", {})
+
+        if interest_region_crop.get('need_crop'):  # 是否需要兴趣区域裁剪
+
+            if interest_region_crop['size_paddingUnit'] == 'pixel':
+                interest_width = interest_region_crop["width"]
+                interest_height = interest_region_crop["height"]
+            else:
+                interest_width = interest_region_crop["width"] * slot_width
+                interest_height = interest_region_crop["height"] * slot_height
+
+            if interest_region_crop['origin_paddingUnit'] == 'pixel':
+                interest_top = interest_region_crop["top"]
+                interest_left = interest_region_crop["left"]
+            else:
+                interest_top = interest_region_crop["top"] * slot_width
+                interest_left = interest_region_crop["left"] * slot_height
+
+            height_scale = interest_height / ((y2 - y1) * img.height)
+            transform_img_width = img.width * height_scale
+            transform_img_height = img.height * height_scale
+
+            x_center = left + (right - left) / 2  # x 轴的居中
+            if transform_img_width >= slot_width and transform_img_height >= slot_height:  # 槽的上下左右都能被满足
+                scale_x = scale_y = height_scale
+                y = y1
+                x = self._alternative_interest_center(x_center, slot_width, transform_img_width, interest_left,
+                                                      interest_width)
+
+                self.si.top_and_bottom_crop_result(True)  # 上下裁剪点都存在
+
+            elif transform_img_height >= slot_height and transform_img_width < slot_width:
+                width_scale = slot_width / img.width
+                y = y1
+                x = 0
+                scale_x = scale_y = width_scale
+
+                self.si.top_and_bottom_crop_result(False)  # 上裁剪点存在
+            else:
+                if retain_x_or_y == "x":
+
+                    min_top_ratio = self.__divide(interest_top, y1 * img.height)
+
+                    min_bottom_ratio = self.__divide(slot_height - interest_top, (1 - y1) * img.height)
+
+                    if left == 0.5 and right == 0.5:  # 图片能放在slot位内 human 会被裁掉
+                        min_img_left_ratio = self.__divide(0.5 * img.width, interest_left + interest_width / 2)
+                        min_img_right_ratio = self.__divide(0.5 * img.width, slot_width-interest_left-interest_width/2)
+
+                        max_ratio = max(min_top_ratio, min_bottom_ratio, min_img_left_ratio, min_img_right_ratio)
+                        # 换算 回 原图的x
+                        x = ((interest_left + interest_width / 2) * max_ratio) / slot_width
+
+                    else:  # human 放在 兴趣区域内
+                        min_human_left_ratio = self.__divide(interest_left, left * img.width)
+
+                        min_human_right_ratio = self.__divide(slot_width-interest_left-interest_width, (1 - right)*img.width)
+
+                        max_ratio = max(min_top_ratio, min_bottom_ratio, min_human_left_ratio, min_human_right_ratio)
+
+                        # 换算 回 原图的x
+                        x = ((interest_left + interest_width / 2) * max_ratio - (
+                                    (right - left) / 2 * img.width)) / slot_width
+
+                    y = y1
+                    scale_x = scale_y = max_ratio
+                    self.si.top_and_bottom_crop_result(False)  # 上裁剪点存在
+
+                elif retain_x_or_y == "interest_y":
+
+                    min_top_ratio = self.__divide(interest_top, y1 * img.height)
+                    min_bottom_ratio = self.__divide(slot_height - interest_top, (1 - y1) * img.height)
+                    max_ratio = max(min_top_ratio, min_bottom_ratio)
+
+                    transform_img_width = img.width * max_ratio
+                    if transform_img_width >= slot_width:
+                        x_center = left + (right - left) / 2  # x 轴的居中
+
+                        x = self._alternative_interest_center(x_center, slot_width, transform_img_width, interest_left,
+                                                              interest_width)
+                        scale_x = scale_y = max_ratio
+                        if max_ratio == min_bottom_ratio:
+                            self.si.top_and_bottom_crop_result(True)  # 上下裁剪点存在
+                        else:
+                            self.si.top_and_bottom_crop_result(False)  # 上裁剪点存在
+
+                    else:
+                        width_scale = slot_width / img.width
+                        x = 0
+                        scale_x = scale_y = width_scale
+                        self.si.top_and_bottom_crop_result(False)  # 上裁剪点存在
+                    y = y1
+
+                else:
+                    height_scale = slot_height / img.height
+
+                    transform_img_width = img.width * height_scale
+
+                    if transform_img_width >= slot_width:
+                        x_center = left + (right - left) / 2  # x 轴的居中
+
+                        x = self._alternative_interest_center(x_center, slot_width, transform_img_width, interest_left,
+                                                              interest_width)
+                        scale_x = scale_y = height_scale
+                    else:
+                        width_scale = slot_width / img.width
+                        x = 0
+                        scale_x = scale_y = width_scale
+                    y = y1
+
+                    self.si.top_and_bottom_crop_result(True)  # 上下裁剪点存在
+
+            self.si.main_part_offset(x * img.width + interest_left / scale_x + interest_width / scale_x / 2,
+                                     x_center * img.width, 'x')
+
+            crop_x = x * img.width
+            crop_y = y * img.height
+            crop_width = slot_width / scale_x
+            crop_height = slot_height / scale_x
+            amend_layout_height = 0
+
+        elif crop_method == 'w':  # 自适应裁剪，slot宽不变,高可以改变
             # 保留上下 y ，还是保留左右居中 x
             retain_x_or_y = crop_info.get(RETAIN_X_OR_Y, "y")
             if retain_x_or_y == 'y':
@@ -298,8 +429,7 @@ class ImageCrop:
             else:
                 width_scale = slot_width / img.width
                 y = y1
-                # 保留上下，还是保留左右  human_center human_and_cloth brand
-                retain_x_or_y = crop_info.get(RETAIN_X_OR_Y, "x")
+
                 if retain_x_or_y == "y":
                     scale_x = scale_y = width_scale
                     x = x1
@@ -656,6 +786,30 @@ class ImageCrop:
             flag_index += 1
         return crop_bottom['bottom_site'], crop_bottom_name
 
+    def _alternative_interest_center(self, center, slot_height_or_width, transform_height_or_width,
+                                     interest_left_or_top,
+                                     interest_slot_height_or_width):
+        center_ = center * transform_height_or_width
+        # 在图片的范围内
+        tag_1 = (center_ - interest_slot_height_or_width / 2 - interest_left_or_top) >= 0
+        tag_2 = (center_ + interest_slot_height_or_width / 2 + (
+                slot_height_or_width - interest_slot_height_or_width - interest_left_or_top)) <= transform_height_or_width
+
+        if tag_1 and tag_2:  # 1 2 都有足够 预留范围
+            x_or_y = center_ - interest_slot_height_or_width / 2 - interest_left_or_top
+        elif tag_1 and not tag_2:  # 1范围够 2范围不够
+            x_or_y = transform_height_or_width - interest_slot_height_or_width
+        elif tag_2 and not tag_1:  # 1范围不够 2范围够
+            x_or_y = 0
+        else:  # 未知情况
+            x_or_y = 0
+        x_or_y = x_or_y / transform_height_or_width
+
+        if not (tag_1 and tag_2):
+            self.si.extension()
+
+        return x_or_y
+
     def _alternative_center(self, center, slot_height_or_width, transform_height_or_width):
         center_ = center * transform_height_or_width
         # 在图片的范围内
